﻿using System.Collections.Specialized;
using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Rosieks.Web.Flow.Tests
{
    [TestClass]
    public class ActionNameBinderTest
    {
        [TestMethod]
        public void ActionNameFromButtonTest()
        {
            var formCollection = new NameValueCollection
            {
                { "ActionName", "next"},
                { "ActionName", "prev"},
                { "ActionName.next", "Go Next >>" }
            };

            var controllerContext = new ControllerContext();
            var bindingContext = new ModelBindingContext
            {
                ValueProvider = new NameValueCollectionValueProvider(formCollection, null),
                ModelMetadata = ModelMetadataProviders.Current.GetMetadataForType(null, typeof(string)),
            };

            var binder = new ActionNameBinder();

            var actionName = binder.BindModel(controllerContext, bindingContext);

            Assert.AreEqual("next", actionName);
        }

        [TestMethod]
        public void ActionNameFromButtonTest_NoActionButtonClick()
        {
            var formCollection = new NameValueCollection
            {
                { "ActionName", "next"},
                { "ActionName", "prev"},
            };

            var controllerContext = new ControllerContext();
            var bindingContext = new ModelBindingContext
            {
                ValueProvider = new NameValueCollectionValueProvider(formCollection, null),
                ModelMetadata = ModelMetadataProviders.Current.GetMetadataForType(null, typeof(string)),
            };

            var binder = new ActionNameBinder();

            var actionName = binder.BindModel(controllerContext, bindingContext);

            Assert.AreEqual(null, actionName);
        }

        [TestMethod]
        public void ActionNameFromImageButtonTest()
        {
            var formCollection = new NameValueCollection
            {
                { "ActionName", "next"},
                { "ActionName", "prev"},
                { "ActionName.next.x", "10" },
                { "ActionName.next.y", "90" },
            };

            var controllerContext = new ControllerContext();
            var bindingContext = new ModelBindingContext
            {
                ValueProvider = new NameValueCollectionValueProvider(formCollection, null),
                ModelMetadata = ModelMetadataProviders.Current.GetMetadataForType(null, typeof(string)),
            };

            var binder = new ActionNameBinder();

            var actionName = binder.BindModel(controllerContext, bindingContext);

            Assert.AreEqual("next", actionName);
        }
    }
}
