﻿using System;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Rosieks.Web.Flow.Metadata;
using Rosieks.Web.Flow.Tests.Helpers;

namespace Rosieks.Web.Flow.Tests
{
    [TestClass]
    public class FlowControllerTest
    {
        private WebflowModel model = new WebflowModel("myFlow")
        {
            States = new[]
            {
                new ViewState { Name = "Step1", ViewPath = "~/Step1.aspx" },
                new ViewState { Name = "Step2", ViewPath = "~/Step2.aspx" },
            }
        };

        //[TestMethod]
        public void InitWorkflow()
        {
            string flowName = "flow1";

            var webflowContextProvider = new Mock<IWebflowContextProvider>();
            var controller = new FlowController
            {
                //WebflowContextProvider = webflowContextProvider.Object,
            };

            webflowContextProvider
                .Setup(m => m.GetContext(It.Is<ControllerContext>(c => c.Equals(controller.ControllerContext)), flowName))
                .Returns((WebflowContext)null);

            //var result = controller.OnState("flow1") as ViewResult;

            //Assert.IsNotNull(result);
            //Assert.AreEqual(expectedViewName, result.ViewName);

            //webflowContextProvider
            //    .Verify(m => m.SaveContext(
            //        It.Is<ControllerContext>(c => c.Equals(controller.ControllerContext)),
            //        It.Is<WebflowContext>(c => c.CurrentState.Name == "Step1"),
            //        It.Is<string>(n => n == flowName))
            //    );

        }

        [TestMethod]
        public void OnStateSubmit()
        {
            WebflowContext webflowContext = null;
            var state = new ViewState { Name = "Step1" };
            var httpContext = ContextHelper.CreateHttpContext().WithWebflow(state, new SampleModel(), ref webflowContext);
            var flowController = new FlowTestController(httpContext.Object);

            var form = new FormCollection
            {
                { "Firstname", "Sławek"},
                { "Lastname", "Rosiek" },
            };

            var result = flowController.OnState(null, "next", form) as WebflowResult;
            var model = webflowContext.Model as SampleModel;

            Assert.IsInstanceOfType(result, typeof(WebflowResult));
            Assert.AreEqual("next", result.ActionName);
            Assert.AreEqual("Sławek", model.Firstname);
            Assert.AreEqual("Rosiek", model.Lastname);
            Assert.AreEqual(null, model.Email);
        }
    }

    public class FlowTestController : FlowController
    {
        public FlowTestController(HttpContextBase context)
        {
            var requestContext = new RequestContext(context, new System.Web.Routing.RouteData());

            this.Initialize(requestContext);
        }

        public Func<object> Step1Data { get; set; }

        public object PrepareDataForStep1()
        {
            return this.Step1Data();
        }
    }
}
