﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Web;
using Moq;
using Rosieks.Web.Flow.Metadata;

namespace Rosieks.Web.Flow.Tests.Helpers
{
    internal static class ContextHelper
    {
        public static Mock<HttpContextBase> CreateHttpContext()
        {
            var httpContext = new Mock<HttpContextBase>();
            httpContext.Setup(c => c.Items).Returns(new Dictionary<object, object>());

            return httpContext;
        }

        #region Webflow helpers

        public static Mock<HttpContextBase> WithWebflowContext(this Mock<HttpContextBase> httpContext, WebflowContext webflowContext)
        {
            return httpContext.WithWebflowContext(webflowContext, null);
        }

        public static Mock<HttpContextBase> WithWebflowContext(this Mock<HttpContextBase> httpContext, WebflowContext webflowContext, string name)
        {
            var httpContextObject = httpContext.Object;

            httpContextObject.Items["__CurrentWebflow__"] = webflowContext;
            httpContextObject.Items["__CurrentWebflowName__"] = name;

            return httpContext;
        }

        public static Mock<HttpContextBase> WithWebflow(this Mock<HttpContextBase> httpContext, State state, object model)
        {
            WebflowContext webflowContext = null;

            return httpContext.WithWebflow(state, model, ref webflowContext);
        }

        public static Mock<HttpContextBase> WithWebflow(this Mock<HttpContextBase> httpContext, State state, object model, ref WebflowContext webflowContext)
        {
            webflowContext = new WebflowContext()
            {
                CurrentState = state,
                Model = model,
            };

            return httpContext.WithWebflowContext(webflowContext);
        }

        public static Mock<HttpContextBase> WithWebflow(this Mock<HttpContextBase> httpContext, State state, WebflowModel webflow, ref WebflowContext webflowContext)
        {
            webflowContext = new WebflowContext
            {
                CurrentState = state,
                Webflow = webflow,
            };

            return httpContext.WithWebflowContext(webflowContext);
        }

        public static Mock<HttpContextBase> WithWebflow(this Mock<HttpContextBase> httpContext, State state, WebflowModel webflow)
        {
            WebflowContext webflowContext = null;

            return httpContext.WithWebflow(state, webflow, ref webflowContext);
        }

        #endregion

        #region HttpRequest helpers

        public static Mock<HttpContextBase> WithRequest(this Mock<HttpContextBase> httpContext, Action<Mock<HttpRequestBase>> setupRequest = null)
        {
            var httpRequest = new Mock<HttpRequestBase>();
            httpRequest.Setup(r => r.Headers).Returns(new NameValueCollection());
            if (setupRequest != null)
            {
                setupRequest(httpRequest);
            }

            httpContext.Setup(c => c.Request).Returns(httpRequest.Object);

            return httpContext;
        }

        public static Mock<HttpRequestBase> AsAjax(this Mock<HttpRequestBase> httpRequest)
        {
            httpRequest.Object.Headers["X-Requested-With"] = "XMLHttpRequest";

            return httpRequest;
        }

        #endregion

        #region HttpResponse helpers

        public static Mock<HttpContextBase> WithResponse(this Mock<HttpContextBase> httpContext, Action<Mock<HttpResponseBase>> setupResponse)
        {
            Mock<HttpResponseBase> httpResponse = null;

            return httpContext.WithResponse(setupResponse, ref httpResponse);
        }

        public static Mock<HttpContextBase> WithResponse(this Mock<HttpContextBase> httpContext, Action<Mock<HttpResponseBase>> setupResponse, ref Mock<HttpResponseBase> httpResponse)
        {
            httpResponse = new Mock<HttpResponseBase>();

            if (setupResponse != null)
            {
                setupResponse(httpResponse);
            }

            httpContext.Setup(c => c.Response).Returns(httpResponse.Object);

            return httpContext;
        }

        public static Mock<HttpResponseBase> WithCache(this Mock<HttpResponseBase> httpResponse, ref Mock<HttpCachePolicyBase> httpCachePolicy)
        {
            httpCachePolicy = new Mock<HttpCachePolicyBase>();

            httpResponse.Setup(r=> r.Cache).Returns(httpCachePolicy.Object);

            return httpResponse;
        }

        #endregion
    }
}
