﻿using System.Collections.Specialized;
using System.Web;
using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Rosieks.Web.Flow.Tests.Helpers;

namespace Rosieks.Web.Flow.Tests
{
    [TestClass]
    public class HiddenFieldWebflowContextProviderTest
    {
        [TestMethod]
        public void GetContext_NoContextInRequest_ReturnNull()
        {
            string webflowName = "myWebflow";

            // Initialize dependencies
            var formatter = new Mock<LosFormatterBase>();

            // Initialize method parameters
            var httpContext = new Mock<HttpContextBase>();
            var httpRequest = new Mock<HttpRequestBase>();
            var controllerContext = new ControllerContext();
            var formCollection = new NameValueCollection();

            httpRequest.SetupGet(c => c.Form).Returns(formCollection);
            httpContext.SetupGet(c => c.Request).Returns(httpRequest.Object);
            controllerContext.HttpContext = httpContext.Object;

            // Create tested object
            var webflowContextProvider = new HiddenFieldWebflowContextProvider(formatter.Object);

            // Test method
            var actualContext = webflowContextProvider.GetContext(controllerContext, webflowName);

            // Check results
            Assert.IsNull(actualContext);
        }

        [TestMethod]
        public void GetContext_ContextInRequest_ReturnDeserializedContextFromRequest()
        {
            string webflowName = "myWebflow";
            string expectedCurrentStateName = "step2";
            SampleModel m = new SampleModel();

            // Initialize webflow context
            var webflowContext = new WebflowContext
            {
                CurrentStateName = "step2",
                Model = m,
            };

            // Initialize dependencies
            var formatter = new Mock<LosFormatterBase>();

            formatter.Setup(f => f.Deserialize("abc")).Returns(webflowContext);

            // Initialize method parameters
            var httpContext = new Mock<HttpContextBase>();
            var httpRequest = new Mock<HttpRequestBase>();
            var controllerContext = new ControllerContext();
            var formCollection = new NameValueCollection();

            httpRequest.Setup(c => c[It.IsAny<string>()]).Returns<string>(s => formCollection[s]);
            httpContext.Setup(c => c.Request).Returns(httpRequest.Object);
            controllerContext.HttpContext = httpContext.Object;
            formCollection["__WebflowContext__"] = "abc";

            // Create tested object
            var webflowContextProvider = new HiddenFieldWebflowContextProvider(formatter.Object);

            // Test method
            var actualContext = webflowContextProvider.GetContext(controllerContext, webflowName);

            // Check results
            Assert.AreEqual(webflowContext, actualContext);
            Assert.AreEqual(m, actualContext.Model);
        }

        [TestMethod]
        public void SetContext_CurrentElementIsNotNull_ContextSerializedToHiddenField()
        {
            Assert.Inconclusive();
        }

        [TestMethod]
        public void SetContext_CurrentElementIsNull_NoContextStored()
        {
            Assert.Inconclusive();
        }
    }
}
