﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using System.Web.Routing;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Rosieks.Web.Flow.Tests
{
    [TestClass]
    public class RouteDataTest
    {
        [TestMethod]
        public void Register_RouteWithThisNameDoesntExists()
        {
            string flowName = "myWebflow";
            var routes = new RouteCollection();
            var routeData = new RouteData
            {
                RouteName = "route",
                Values = new RouteValueDictionary(new { controller = "Home", action = "Index" }),
                Url = "url",
            };

            routeData.Register(routes, flowName);

            Route route = routes["route"] as Route;

            Assert.AreEqual("url", route.Url);
            Assert.AreEqual(flowName, route.Defaults[RouteData.WebflowParamName]);
            Assert.AreEqual("Home", route.Defaults["controller"]);
            Assert.AreEqual("Index", route.Defaults["action"]);
            Assert.IsInstanceOfType(route.RouteHandler, typeof(MvcRouteHandler));
        }

        [TestMethod]
        public void Register_RouteWithoutControllerAndIndex()
        {
            string flowName = "myWebflow";
            var routes = new RouteCollection();
            var routeData = new RouteData
            {
                RouteName = "route",
                Url = "url",
            };

            routeData.Register(routes, flowName);

            Route route = routes["route"] as Route;

            Assert.AreEqual("url", route.Url);
            Assert.AreEqual(flowName, route.Defaults[RouteData.WebflowParamName]);
            Assert.AreEqual("Flow", route.Defaults["controller"]);
            Assert.AreEqual("OnState", route.Defaults["action"]);
            Assert.AreEqual(typeof(FlowController).Namespace, ((IEnumerable<string>)route.DataTokens["Namespaces"]).First());
            Assert.IsInstanceOfType(route.RouteHandler, typeof(MvcRouteHandler));
        }

        [TestMethod]
        public void Register_RouteWithThisNameAlreadyExistsAndHasTheSameUrl()
        {
            string flowName = "myWebflow";
            var routes = new RouteCollection();

            var routeData = new RouteData
            {
                RouteName = "route",
                Values = new RouteValueDictionary(new { controller = "Home", action = "Index" }),
                Url = "url",
            };

            routes.MapRoute("route", "url", new { controller = "Home2", action = "Index2" });
            routeData.Register(routes, flowName);

            Route route = routes["route"] as Route;

            Assert.AreEqual("url", route.Url);
            Assert.AreEqual(flowName, route.Defaults[RouteData.WebflowParamName]);
            Assert.AreEqual("Home2", route.Defaults["controller"]);
            Assert.AreEqual("Index2", route.Defaults["action"]);
            Assert.IsInstanceOfType(route.RouteHandler, typeof(MvcRouteHandler));
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void Register_RouteWithThisNameAlreadyExistsButHasDifferentUrl()
        {
            string flowName = "myWebflow";
            var routes = new RouteCollection();

            var routeData = new RouteData
            {
                RouteName = "route",
                Values = new RouteValueDictionary(new { controller = "Home", action = "Index" }),
                Url = "url",
            };

            routes.MapRoute("route", "url2");
            routeData.Register(routes, flowName);

            Route route = routes["route"] as Route;

            Assert.AreEqual("url", route.Url);
            Assert.AreEqual(flowName, route.Defaults[RouteData.WebflowParamName]);
            Assert.AreEqual("Home", route.Defaults["controller"]);
            Assert.AreEqual("Index", route.Defaults["action"]);
            Assert.IsInstanceOfType(route.RouteHandler, typeof(MvcRouteHandler));
        }
    }
}
