﻿using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Rosieks.Web.Flow.Metadata;

namespace Rosieks.Web.Flow.Tests
{
    [TestClass]
    public class TempDataWebflowContextProviderTest
    {
        [TestMethod]
        public void GetContext_ContextInTempData_ReturnDeserializedContextFromTempData()
        {
            string webflowName = "myWebflow";

            var webflowContext = new WebflowContext
            {
                CurrentStateName = "step1",
            };

            var controller = new Mock<ControllerBase>();
            var controllerContext = new ControllerContext();

            controller.Object.TempData = new TempDataDictionary { { "__WebflowContext_myWebflow__", webflowContext } };
            controllerContext.Controller = controller.Object;

            var webflowContextProvider = new TempDataWebflowContextProvider();

            var actualContext = webflowContextProvider.GetContext(controllerContext, webflowName);

            Assert.AreEqual(webflowContext, actualContext);
        }

        [TestMethod]
        public void GetContext_NoContextInTempData_ReturnNull()
        {
            string webflowName = "myWebflow";

            var controller = new Mock<ControllerBase>();
            var controllerContext = new ControllerContext();

            controller.Object.TempData = new TempDataDictionary();
            controllerContext.Controller = controller.Object;

            var webflowContextProvider = new TempDataWebflowContextProvider();

            var actualContext = webflowContextProvider.GetContext(controllerContext, webflowName);

            Assert.IsNull(actualContext);
        }

        [TestMethod]
        public void SetContext_CurrentElementIsNotNull_ContextStoredToTempData()
        {
            string webflowName = "myWebflow";

            var state = new ViewState { Name = "state1" };
            var webflowModel = new Mock<WebflowModel>();
            var webflowContext = new WebflowContext { CurrentState = state, Webflow = webflowModel.Object };
            var tempData = new TempDataDictionary();

            var controller = new Mock<ControllerBase>();
            var controllerContext = new Mock<ControllerContext>();

            webflowModel.Object.Name = webflowName;
            controller.Object.TempData = tempData;
            controllerContext.Setup(c => c.Controller).Returns(controller.Object);

            var webflowContextProvider = new TempDataWebflowContextProvider();

            webflowContextProvider.SetContext(controllerContext.Object, webflowContext);

            var contextData = (WebflowContext)tempData["__WebflowContext_myWebflow__"];

            Assert.AreEqual("state1", contextData.CurrentStateName);
        }

        [TestMethod]
        public void DeleteContext_ContextExistInTempData_ContextRemovedFromTempData()
        {
            string webflowName = "myWebflow";

            var webflowModel = new Mock<WebflowModel>();
            var webflowContext = new WebflowContext { Webflow = webflowModel.Object };
            var tempData = new TempDataDictionary { { "__WebflowContext_myWebflow__", webflowContext } };

            var controller = new Mock<ControllerBase>();
            var controllerContext = new ControllerContext();

            webflowModel.Object.Name = webflowName;
            controller.Object.TempData = tempData;
            controllerContext.Controller = controller.Object;

            var webflowContextProvider = new TempDataWebflowContextProvider();

            webflowContextProvider.DeleteContext(controllerContext, webflowContext);

            Assert.IsFalse(tempData.ContainsKey("__WebflowContext_myWebflow__"));
        }
    }
}
