﻿
using System.Web;
using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Rosieks.Web.Flow.Metadata;
using Rosieks.Web.Flow.Tests.Helpers;
namespace Rosieks.Web.Flow.Tests
{
    [TestClass]
    public class ViewStateResultAdapterTest
    {
        [TestMethod]
        public void CreateResult()
        {
            string expectedViewName = "~/Views/Flow/Step1.cshtml";
            string stateName = "state1";

            var webflowModel = new object();
            var model = new object();
            Mock<HttpCachePolicyBase> httpCachePolicy = null;

            // Initialize target
            var adapter = new ViewStateResultAdapter();

            // Initialize parameters
            var viewState = new ViewState
            {
                Name = stateName,
                ViewPath = expectedViewName,
            };

            var controllerContext = new ControllerContext();
            controllerContext.Controller = new FakeController();
            controllerContext.Controller.ViewData["ViewDataTest"] = "ViewDataContent";
            controllerContext.Controller.ViewData.Model = model;
            controllerContext.Controller.TempData["TempDataTest"] = "TempDataContent";
            controllerContext.HttpContext = ContextHelper.CreateHttpContext().WithWebflow(viewState, webflowModel).WithResponse(r => r.WithCache(ref httpCachePolicy)).Object;

            // Test
            ViewResult actual = adapter.CreateResult(viewState, controllerContext) as ViewResult;

            // Verify
            Assert.AreEqual(expectedViewName, actual.ViewName);
            Assert.AreEqual(webflowModel, actual.ViewData["__WebflowModel_ViewData_Key"]);
            Assert.AreEqual("ViewDataContent", actual.ViewData["ViewDataTest"]);
            Assert.AreEqual("TempDataContent", actual.TempData["TempDataTest"]);
            httpCachePolicy.Verify(c => c.SetCacheability(HttpCacheability.NoCache));
        }
    }
}
