﻿using System.Web;
using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Rosieks.Web.Flow.Tests.Helpers;

namespace Rosieks.Web.Flow.Tests
{
    /// <summary>
    /// Summary description for WebflowAccessableAttributeTest
    /// </summary>
    [TestClass]
    public class WebflowAccessableAttributeTest
    {
        public WebflowAccessableAttributeTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void OnActionExecuting_Load_Success()
        {
            var controller = new FakeController();
            var filterContext = new ActionExecutingContext();
            var manager = new Mock<IWebflowManager>();

            filterContext.Controller = controller;

            var target = new WebflowAccessableAttribute();
            target.Manager = manager.Object;

            target.OnActionExecuting(filterContext);

            manager.Verify(m => m.Load(controller.ControllerContext, null));
        }

        [TestMethod]
        public void OnActionExecuting_LoadByName_Success()
        {
            var controller = new FakeController();
            var filterContext = new ActionExecutingContext();
            var manager = new Mock<IWebflowManager>();

            filterContext.Controller = controller;

            var target = new WebflowAccessableAttribute();
            target.Manager = manager.Object;
            target.Name = "mywebflow";

            target.OnActionExecuting(filterContext);

            manager.Verify(m => m.Load(controller.ControllerContext, "mywebflow"));
        }

        [TestMethod]
        public void OnActionExecuting_LoadByParamName_Success()
        {
            var controller = new FakeController();
            var filterContext = new ActionExecutingContext();
            var manager = new Mock<IWebflowManager>();
            var httpContext = new Mock<HttpContextBase>();
            var httpRequest = new Mock<HttpRequestBase>();

            httpRequest.Setup(r => r[It.Is<string>(s => s == "name")]).Returns("mywebflow");
            httpContext.Setup(c => c.Request).Returns(httpRequest.Object);
            filterContext.Controller = controller;
            filterContext.HttpContext = httpContext.Object;

            var target = new WebflowAccessableAttribute();
            target.Manager = manager.Object;
            target.NameByParam = "name";

            target.OnActionExecuting(filterContext);


            manager.Verify(m => m.Load(controller.ControllerContext, "mywebflow"));
        }

        [TestMethod]
        public void OnActionExecuted_Save_Success()
        {
            var controller = new FakeController();
            var filterContext = new ActionExecutedContext();
            var manager = new Mock<IWebflowManager>();

            filterContext.Controller = controller;

            var target = new WebflowAccessableAttribute();
            target.Manager = manager.Object;

            target.OnActionExecuted(filterContext);

            manager.Verify(m => m.Save(controller.ControllerContext));
        }
    }
}
