﻿using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Rosieks.Web.Flow.Metadata;
using Rosieks.Web.Flow.Tests.Helpers;

namespace Rosieks.Web.Flow.Tests
{
    [TestClass]
    public class WebflowResultTest
    {
        [TestMethod]
        public void OnExecute()
        {
            var transitionResultAdapter = new Mock<IWebflowElementResultAdapter>();
            var result = new Mock<ActionResult>();
            var transition = new Transition { On = "myAction" };
            var webflowContext = new WebflowContext() { Model = new SampleModel() };
            var httpContext = ContextHelper.CreateHttpContext().WithWebflowContext(webflowContext);
            var controllerContext = new ControllerContext();

            transitionResultAdapter.Setup(t => t.CreateResult(It.IsAny<Transition>(), It.IsAny<ControllerContext>())).Returns(result.Object);

            var webflowResult = new WebflowResult("myAction");

            controllerContext.HttpContext = httpContext.Object;
            webflowContext.CurrentElement = transition;

            WebflowElementResultProvider.Adapters[transition.GetType()] = () => transitionResultAdapter.Object;

            webflowResult.ExecuteResult(controllerContext);

            transitionResultAdapter.Verify(v => v.CreateResult(It.Is<Transition>(t => t == transition), It.IsAny<ControllerContext>()));
            result.Verify(v => v.ExecuteResult(It.IsAny<ControllerContext>()));
        }
    }
}
