﻿using System;
using System.ComponentModel.DataAnnotations;
using Rosieks.Web.Flow;
using Rosieks.Web.Flow.Fluent;
using Rosieks.Web.Flow.Metadata;

namespace UseCases.Models
{
    public class RecruitmentFlowModel
    {
        public PersonData PersonData { get; set; }
    }

    public class PersonData
    {
        [Required]
        public string Firstname { get; set; }
        public string Lastname { get; set; }
        public string BirthPlace { get; set; }

        [Required]
        public DateTime BirthDate { get; set; }
        public string FatherName { get; set; }
        public string MotherName { get; set; }
        public string Email { get; set; }
        public string Phone { get; set; }
        public string Msisdn { get; set; }

        public Address HomeAddress { get; set; }
        public Address MailingAddress { get; set; }
    }

    public class Address
    {
        public string Street { get; set; }
        public string HomeNo { get; set; }
        public string AppartamentNo { get; set; }
        public string PostCode { get; set; }
        public string City { get; set; }
    }

    public class RecruitmentWebflow : WebflowModel<RecruitmentFlowModel>
    {
        public RecruitmentWebflow()
            : base("Recruitment")
        {
            this.RegisterStates(s => s
                .State<ViewState>("step1")
                    .IsFirstStep()
                    .WithView("~/Views/Recruitment/Step1.cshtml")
                    .WithRoutes("RecruitmentWizard-Step1", new { step = 1, action = "Step1", controller = "Recruitment" }, "rekrutacja/1")
                    .WithTransisions(t => t.Transition("next", "step2"))
                .State<ViewState>("step2")
                    .WithView("~/Views/Recruitment/Step2.cshtml")
                    .WithRoutes("RecruitmentWizard-Step", new { step = 2 }, "rekrutacja/{step}")
                    .WithTransisions(t => t
                        .Transition("next", "step3")
                        .Transition("back", "step1").WithoutValidation()
                    )
                .State<ViewState>("step3")
                    .WithView("~/Views/Recruitment/Step3.cshtml")
                    .WithRoutes("RecruitmentWizard-Step", new { step = 3 })
                    .WithTransisions(t => t.Transition("submit", "step4"))
                .State<ViewState>("step4")
                    .WithView("~/Views/Recruitment/Step4.cshtml")
                    .WithRoutes("RecruitmentWizard-Step", new { step = 4 })
                    .IsEndStep()
            );

            HashtableWebflowCatalog.Instance.Register(this);
        }
    }
}